<?php

/** 
 * Joomla.Plugin - Content - NetzSinn Address2Coordinates
 * ------------------------------------------------------------------------
 * @package     Content - NetzSinn Address2Coordinates
 * @author      Netzsinn.de
 * @copyright   2023 netzsinn.de
 * @license     GNU/GPLv3 <http://www.gnu.org/licenses/gpl-3.0.de.html>
 * @link        https://netzsinn.de
 * ------------------------------------------------------------------------
 */

// no direct access
defined('_JEXEC') or die;

use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\CMS\Form\Form;
use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;


/**
 * 
 */
class PlgContentNetzsinn_address2coordinates extends CMSPlugin
{

	/**
	 * Application object
	 *
	 * @var    \Joomla\CMS\Application\CMSApplication
	 * @since  4.0.0
	 */
	protected $app;

	/** 
	 * This is called before the Model is called. Here we change the input parameters.
	 * 
	 * @see https://joomla.stackexchange.com/questions/32326/plugin-to-manipulate-form-data-onbeforecontentsave
	 * @throws Exception
	 */
	public function onContentNormaliseRequestData($context, $data, Form $form)
	{
		if (!in_array($context, array('com_content.article'))) {
			return true;
		}

		// get the input data
		$is_debug = $this->params->get('is_debug');
		$google_api_key = $this->params->get('google_api_key');
		$countries = $this->params->get('countries');

		// Get the settings valid for the given URL ... or leave the Plugin
		$settings = $this->params->get('settings');
		if ($settings == null) {
			return true;
		}

		// Skip if we do not have any fields
		if (!is_array($data->com_fields)) {
			return true;
		}

		foreach ($settings as $setting) {

			$skip = array_key_exists($setting->field_skip, $data->com_fields) ? ($setting->is_skip == 1 ? $data->com_fields[$setting->field_skip] : null) : null;
			$street = array_key_exists($setting->field_street, $data->com_fields) ? ($setting->is_field_street == 1 ? $data->com_fields[$setting->field_street] : null) : null;
			$zip = array_key_exists($setting->field_zip, $data->com_fields) ? ($setting->is_field_zip == 1 ? $data->com_fields[$setting->field_zip] : null) : null;
			$city = array_key_exists($setting->field_city, $data->com_fields) ? ($setting->is_field_city == 1 ? $data->com_fields[$setting->field_city] : null)  : null;
			$country = array_key_exists($setting->field_country, $data->com_fields) ? ($setting->is_field_country == 1 ? $data->com_fields[$setting->field_country] : null) : null;

			$skip_overwrite = ($setting->is_skip == 1 && $skip == 1) ? true : false;
			$is_resolve_countries = $setting->is_resolve_countries;

			if (strlen(trim($city)) > 0) {
				$address = '';

				if (strlen(trim($street)) > 0) $address .= ($street);
				if (strlen(trim($zip)) > 0) $address .= ', ' .  ($zip);
				if (strlen(trim($zip)) > 0 && strlen(trim($city)) > 0) $address .= ' ' . ($city);
				if ($zip == null && strlen(trim($city)) > 0) $address .= ', ' . ($city);
				if (strlen(trim($country)) > 0) {
					if ($is_resolve_countries) {
						$landname = $this->_resolveLand($country, $countries);
						$address .= ', ' .  $landname;
					} else {
						$address .= ', ' .  $country;
					}
				}

				$address = trim(ltrim($address, ','));
			} elseif (strlen(trim($city)) > 0)  // just city
			{
				$address = $city;
			}

			$field_lat = array_key_exists($setting->field_lat, $data->com_fields) ? ($setting->is_fields_lat_lng == 1 ? $data->com_fields[$setting->field_lat] : null) : null;
			$field_lng = array_key_exists($setting->field_lng, $data->com_fields) ? ($setting->is_fields_lat_lng == 1 ? $data->com_fields[$setting->field_lng] : null) : null;
			$field_lat_lng = array_key_exists($setting->field_lat_lng, $data->com_fields) ? ($setting->is_field_lat_lng == 1 ? $data->com_fields[$setting->field_lat_lng] : null) : null;

			if (
				strlen(trim($address)) > 3 &&
				((array_key_exists($setting->field_lat, $data->com_fields) &&
					array_key_exists($setting->field_lng, $data->com_fields) &&
					$setting->is_fields_lat_lng == 1) ||
					(array_key_exists($setting->field_lat_lng, $data->com_fields) &&
						$setting->is_field_lat_lng == 1
					))
			) {
				$coordinates = $this->_getCoordinates($address, $google_api_key);

				if (
					array_key_exists($setting->field_lat, $data->com_fields) &&
					array_key_exists($setting->field_lng, $data->com_fields) &&
					$setting->is_fields_lat_lng == 1 &&
					$skip_overwrite === false
				) {
					$data->com_fields[$setting->field_lat] = $coordinates['lat'];
					$data->com_fields[$setting->field_lng] =  $coordinates['lng'];
				}
				if (
					array_key_exists($setting->field_lat_lng, $data->com_fields) &&
					$setting->is_field_lat_lng == 1 &&
					$skip_overwrite === false
				) {
					$data->com_fields[$setting->field_lat_lng] = $coordinates['lat'] . ',' . $coordinates['lng'];
				}

				if ($is_debug) {

					echo '<pre>';
					echo 'Street:  [' . ($setting->is_field_street == 1 ? '<span style="color:green;">Enabled</span>' : '<span style="color:red;">Disabled</span>') . '] ' . $street . ' <br/>';
					echo 'Zip:     [' . ($setting->is_field_zip == 1 ? '<span style="color:green;">Enabled</span>' : '<span style="color:red;">Disabled</span>') . '] ' . $zip . ' <br/>';
					echo 'City:    [' . ($setting->is_field_city == 1 ? '<span style="color:green;">Enabled</span>' : '<span style="color:red;">Disabled</span>') . '] ' . $city . ' <br/>';
					echo 'Country: [' . ($setting->is_field_country == 1 ? '<span style="color:green;">Enabled</span>' : '<span style="color:red;">Disabled</span>') . '] ' . $country . ' <br/>';
					echo 'Skip fetching geodata: [' . ($setting->is_skip == 1 ? '<span style="color:black;">Enabled</span>' : '<span style="color:black;">Disabled</span>') . '] <br/>';
					if ($setting->is_skip == 1) {
						echo 'skip: ' . $skip . '<br/>';
						echo 'is_skip: ' . $setting->is_skip . '<br/>';
						echo 'Skip: ' . ($skip_overwrite === true ? 'Yes' : 'No') . '<br/>';
						if (!is_numeric($skip)) {
							echo '<span style="color:green;">Error: Skip fetching Latitude / Longitude Field must return 0 or 1</span>';
						}
					}

					echo 'Address send to Google: ' . $address . '<br/>';
					echo 'Lat returned from Google: ' . $coordinates['lat'] . '<br/>';
					echo 'Lat returned from Google: ' . $coordinates['lng'] . '<br/>';
					echo 'Overwrite Lat and Lng Field: ' . $setting->is_fields_lat_lng . '<br/>';
					echo 'Overwrite LatLng Field: ' . $setting->is_field_lat_lng . '<br/>';
					echo $setting->field_lat . ': ' . $data->com_fields[$setting->field_lat] . '<br/>';
					echo $setting->field_lng . ': ' . $data->com_fields[$setting->field_lng] . '<br/>';
					echo $setting->field_lat_lng . ': ' . $data->com_fields[$setting->field_lat_lng] . '<br/>';
					echo '</pre>';
					exit;
				}
			} else {
				if ($is_debug) {
					echo '<pre>';
					echo 'Suspend getting latitude and longitude from Google.<br/>';
					echo '</pre>';
				}
			}
		}

		return true;
	}

	private static function _resolveLand($land, $laender)
	{

		$laenderjson = json_decode($laender);

		$landname = '';
		foreach ($laenderjson->data as $l) {
			if ($land == $l->token) {
				$landname = $l->name;
				break;
			}
		}
		return $landname;
	}
	/**
	 * Grab the url of a publicly embeddable video hosted on vimeo
	 *
	 * @param str $video_url The "embed" url of a video
	 *
	 * @return str            The url of the thumbnail, or false if there's an error
	 */
	private static function _getCoordinates($address = '', $api_key = '')
	{
		if (strlen($address) < 5) return false;

		$url = "https://maps.google.com/maps/api/geocode/json?key=" . $api_key . "&address=" . urlencode($address);

		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $url);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		$responseJson = curl_exec($ch);
		curl_close($ch);

		$response = json_decode($responseJson);

		if ($response->status == 'OK') {
			$latitude = $response->results[0]->geometry->location->lat;
			$longitude = $response->results[0]->geometry->location->lng;
		} else {
			echo $response->status;
			var_dump($response);
		}

		return array('lat' => $latitude, 'lng' => $longitude);
	}
}
